@@ -21,7 +21,6 @@ from pathlib import Path
 from typing import List, Optional, Union
 from warnings import warn
 
-import torch
 from omegaconf import DictConfig, ListConfig, OmegaConf
 
 
@@ -35,7 +34,7 @@ def update_input_size_config(config: Union[DictConfig, ListConfig]) -> Union[Dic
         config (Union[DictConfig, ListConfig]): Configurable parameters object
 
     Returns:
-        Configurable parameters with updated values
+        Union[DictConfig, ListConfig]: Configurable parameters with updated values
     """
     # handle image size
     if isinstance(config.dataset.image_size, int):
@@ -56,10 +55,10 @@ def update_nncf_config(config: Union[DictConfig, ListConfig]) -> Union[DictConfi
     """Set the NNCF input size based on the value of the crop_size parameter in the configurable parameters object.
 
     Args:
-        config: Dictconfig: Configurable parameters of the current run.
+        config (Union[DictConfig, ListConfig]): Configurable parameters of the current run.
 
     Returns:
-        Updated configurable parameters in DictConfig object.
+        Union[DictConfig, ListConfig]: Updated configurable parameters in DictConfig object.
     """
     crop_size = config.dataset.image_size
     sample_size = (crop_size, crop_size) if isinstance(crop_size, int) else crop_size
@@ -112,37 +111,10 @@ def update_multi_gpu_training_config(config: Union[DictConfig, ListConfig]) -> U
     return config
 
 
-def update_device_config(config: Union[DictConfig, ListConfig], openvino: bool) -> Union[DictConfig, ListConfig]:
-    """Update XPU Device Config This function ensures devices are configured correctly by the user.
-
-    Args:
-        config (Union[DictConfig, ListConfig]): Input config
-        openvino (bool): Boolean to check if OpenVINO Inference is enabled.
-
-    Returns:
-        Union[DictConfig, ListConfig]: Updated config
-    """
-
-    config.openvino = openvino
-    if openvino:
-        config.trainer.gpus = 0
-
-    if not torch.cuda.is_available():
-        config.trainer.gpus = 0
-
-    if config.trainer.gpus == 0 and torch.cuda.is_available():
-        config.trainer.gpus = 1
-
-    config = update_multi_gpu_training_config(config)
-
-    return config
-
-
 def get_configurable_parameters(
     model_name: Optional[str] = None,
-    model_config_path: Optional[Union[Path, str]] = None,
+    config_path: Optional[Union[Path, str]] = None,
     weight_file: Optional[str] = None,
-    openvino: bool = False,
     config_filename: Optional[str] = "config",
     config_file_extension: Optional[str] = "yaml",
 ) -> Union[DictConfig, ListConfig]:
@@ -150,25 +122,24 @@ def get_configurable_parameters(
 
     Args:
         model_name: Optional[str]:  (Default value = None)
-        model_config_path: Optional[Union[Path, str]]:  (Default value = None)
+        config_path: Optional[Union[Path, str]]:  (Default value = None)
         weight_file: Path to the weight file
-        openvino: Use OpenVINO
         config_filename: Optional[str]:  (Default value = "config")
         config_file_extension: Optional[str]:  (Default value = "yaml")
 
     Returns:
-        Configurable parameters in DictConfig object.
+        Union[DictConfig, ListConfig]: Configurable parameters in DictConfig object.
     """
-    if model_name is None and model_config_path is None:
+    if model_name is None and config_path is None:
         raise ValueError(
             "Both model_name and model config path cannot be None! "
             "Please provide a model name or path to a config file!"
         )
 
-    if model_config_path is None:
-        model_config_path = Path(f"anomalib/models/{model_name}/{config_filename}.{config_file_extension}")
+    if config_path is None:
+        config_path = Path(f"anomalib/models/{model_name}/{config_filename}.{config_file_extension}")
 
-    config = OmegaConf.load(model_config_path)
+    config = OmegaConf.load(config_path)
 
     # Dataset Configs
     if "format" not in config.dataset.keys():
@@ -177,7 +148,10 @@ def get_configurable_parameters(
     config = update_input_size_config(config)
 
     # Project Configs
-    project_path = Path(config.project.path) / config.model.name / config.dataset.name / config.dataset.category
+    project_path = Path(config.project.path) / config.model.name / config.dataset.name
+    if config.dataset.format.lower() in ("btech", "mvtec"):
+        project_path = project_path / config.dataset.category
+
     (project_path / "weights").mkdir(parents=True, exist_ok=True)
     (project_path / "images").mkdir(parents=True, exist_ok=True)
     config.project.path = str(project_path)
@@ -185,10 +159,9 @@ def get_configurable_parameters(
     config.trainer.default_root_dir = str(project_path)
 
     if weight_file:
-        config.weight_file = weight_file
+        config.model.weight_file = weight_file
 
     config = update_nncf_config(config)
-    config = update_device_config(config, openvino)
 
     # thresholding
     if "pixel_default" not in config.model.threshold.keys():
